import re
import requests
import json
from bs4 import BeautifulSoup


def user_timeline(screen_name, output_file='nba.csv'):
    cur_page = 0

    # screen_name, tweet, time, hashtags, cashtags, mentions, emojis, retweet_count, favorite_count
    print 'Starting search user', screen_name
    f = open(output_file, 'a')

    while True:
        cur_page += 1
        # end_tweet_id = ''
        if cur_page == 1:
            url = 'https://twitter.com/' + re.sub('\@', '', screen_name)
            html = requests.request('GET', url).text.encode('ascii', 'ignore')
        else:
            url = 'https://twitter.com/i/profiles/show/' \
                  + re.sub('\@', '', screen_name) \
                  + '/timeline/tweets?include_available_features=1&include_entities=1&max_position='\
                  + end_tweet_id\
                  + '&reset_error_state=false'
            html = json.loads(requests.request('GET', url)._content)['items_html'].strip()
        soup = BeautifulSoup(html, 'html.parser')
        tweet_cardwraps = soup.find_all('li', class_='js-stream-item stream-item stream-item expanding-stream-item ')

        if len(tweet_cardwraps) == 0:
            print 'Done!'
            break

        else:
            print cur_page,
            if cur_page % 10 == 0:
                print '\n'

            for tweet_cardwrap in tweet_cardwraps:
                out = dict()
                hashtags = []
                cashtags = []
                mentions = []
                emojis = []
                try:
                    a_hrefs = tweet_cardwrap.find('div', class_='js-tweet-text-container').p(['a', 'img'])
                    for s in a_hrefs:
                        if s.has_attr('data-expanded-url'):
                            s.extract()
                        elif s.has_attr('data-query-source'):
                            if s.attrs['data-query-source'] == 'cashtag_click':
                                cashtags.append(s.text.strip())
                            elif s.attrs['data-query-source'] == 'hashtag_click':
                                hashtags.append(s.text.strip())
                            # s.extract()
                        elif s.has_attr('data-mentioned-user-id'):
                            mentions.append(s.attrs['data-mentioned-user-id'])
                            # s.extract()
                        elif 'Emoji--forText' in s.attrs['class']:
                            emojis.append('_'.join(s.attrs['title'].split()))
                            s.extract()
                except TypeError:
                    pass
                except AttributeError:
                    continue

                timestamp = tweet_cardwrap.find('a', class_='tweet-timestamp js-permalink js-nav js-tooltip')
                time = timestamp['title']

                text = tweet_cardwrap.find('div', class_='js-tweet-text-container').text
                text = re.sub('[^0-9a-zA-Z\[\]\#\@\$/\.\: ]', ' ', text)
                text = re.sub(' +', ' ', text).strip()

                out['tweet'] =  text
                out['hashtags'] = '|'.join(hashtags)
                out['cashtags'] = '|'.join(cashtags)
                out['mentions'] = '|'.join(mentions)
                out['emojis'] = '|'.join(emojis)
                out['time'] = time
                out['retweet'] = tweet_cardwrap.find('div', class_='ProfileTweet-action ProfileTweet-action--retweet js-toggleState js-toggleRt') \
                    .find('span', class_='ProfileTweet-actionCountForPresentation').text.strip()
                out['favorite'] = tweet_cardwrap.find('div', class_='ProfileTweet-action ProfileTweet-action--favorite js-toggleState') \
                    .find('span', class_='ProfileTweet-actionCountForPresentation').text.strip()

                # screen_name,tweet,time,hashtags,cashtags,mentions,emojis,retweet_count,favorite_count
                f.write(','.join([screen_name, out['tweet'], out['time'],
                                  out['hashtags'], out['cashtags'], out['mentions'], out['emojis'],
                                  out['retweet'], out['favorite']]))
                f.write('\n')

            end_tweet_id = tweet_cardwraps[-1].attrs['data-item-id']

    f.close()


# test
# user_timeline('LeBronJames')
# with open('/Users/cchen224/Downloads/target.html', 'a') as f:
#     f.write(html)


#
# # with open('/Users/cchen224/Downloads/instagram_media.csv', 'r') as i:
# #     csvreader = csv.reader(i)
# #     with open('/Users/cchen224/Downloads/locations.csv', 'a') as o:
# #         csvwriter = csv.writer(o)
# #         [csvwriter.writerow([row[0], row[3], row[-4], row[-2], row[-1]]) for row in csvreader]
# # import random
# # with open('/Users/cchen224/Downloads/geo_sample.csv', 'a') as o:
# #     csvwriter = csv.writer(o)
# #     counter = 0
# #     for index in range(len(location_lat)):
# #         if location_lat[index] != '' and random.uniform(0, 1) < 0.001:
# #             counter += 1
# #             csvwriter.writerow([location_lat[index], location_long[index]])
# #         if counter >= 5000:
# #             break
#
#
#
# import csv
# from datetime import datetime, timedelta
#
# users = []
# create_at = []
# location_lat = []
# location_long = []
# location_id = []
#
# with open('/Users/cchen224/Downloads/locations.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     for row in csvreader:
#         users.append(row[0])
#         create_at.append(row[1])
#         location_id.append(row[2])
#         location_lat.append(row[3])
#         location_long.append(row[4])
# dt = [datetime.strptime(t, '%Y-%m-%d %H:%M:%S') for t in create_at]
#
# # y = 0
# # y_l = 0
# # m = 0
# # m_l = 0
# # w_l = 0
# # w = 0
# # now = datetime.now()
# # for i in range(len(create_at)):
# #     if now - create_at[i] <= timedelta(days=365):
# #         y += 1
# #         if location_lat[i] != '':
# #             y_l += 1
# #         if now - create_at[i] <= timedelta(days=30):
# #             m += 1
# #             if location_lat[i] != '':
# #                 m_l += 1
# #             if now - create_at[i] <= timedelta(days=7):
# #                 w += 1
# #                 if location_lat[i] != '':
# #                     w_l += 1
#
# from geopy.geocoders import Nominatim
# import csv
#
# def get_city_country(lat, long):
#     pair = str(lat) + ', ' + str(long)
#     location = Nominatim().reverse(pair).raw['address']
#     try:
#         city = location['city'].encode('utf-8')
#     except:
#         city = ''
#     try:
#         country = location['country'].encode('utf-8')
#     except:
#         country = ''
#     return city, country
#
# locations = dict()
# with open('/Users/cchen224/Downloads/location_.csv', 'w') as o:
#     csvwriter = csv.writer(o)
#     for i in range(len(users)):
#         if location_lat[i] != '':
#             if location_id[i] not in locations:
#                 city, country = get_city_country(location_lat[i], location_long[i])
#                 locations[location_id[i]] = city, country
#             else:
#                 city, country = locations[location_id[i]]
#         else:
#             city = ''
#             country = ''
#         csvwriter.writerow([users[i], create_at[i], city, country])
#         # user = users[i]
#         # if user not in u:
#         #     u[user] = ''
#         # elif u[user] != '':
#         #     continue
#         # elif location_lat[i] != '':
#         #     city, country = get_city_country(location_lat[i], location_long[i])
#         #     u[user] = country
#
#
#
#
# import matplotlib.pyplot as plt
#
# total = dict()
# for user in users:
#     if user not in total:
#         total[user] = 1
#     else:
#         total[user] += 1
# plt.hist([val for val in total.values()], bins=50)
# plt.hist([val for val in total.values() if val < 50], bins=25)
#
# geo = dict()
# for index in range(len(users)):
#     if users[index] not in geo:
#         geo[users[index]] = 1
#     elif location_lat[index] != '':
#         geo[users[index]] += 1
# plt.hist([float(geo[key]) / total[key] for key in total.keys()], bins=50)
# plt.hist([float(geo[key]) / total[key] for key in total.keys() if total[key] != 1], bins=50)
#
# checkins = dict()
# for index in range(len(users)):
#     if location_lat[index] != '':
#         if users[index] not in checkins:
#             checkins[users[index]] = [(float(location_lat[index]), float(location_long[index]))]
#         else:
#             checkins[users[index]].append((float(location_lat[index]), float(location_long[index])))
#
# def get_rg(list):
#     from geopy.distance import great_circle
#     import math
#     t_lat = 0.
#     t_lon = 0.
#     for item in list:
#         lat, long = item
#         t_lat += lat
#         t_lon += long
#     m_lat = t_lat / len(list)
#     m_lon = t_lon / len(list)
#     d = []
#     for item in list:
#         d.append(great_circle(item, (m_lat, m_lon)).miles)
#         # d.append((item[0], m_lat) ** 2)
#         # d.append((item[1] - m_lon) ** 2)
#     return math.sqrt( sum(d) / len(list) )
# checkins_dist = [get_rg(checkins[key]) for key in checkins.keys()]
# plt.hist(checkins_dist, bins=50, range=[0,80])
# plt.ylabel('frequency')
# plt.xlabel('rg (miles)')
# plt.show()
#
# u_friends = dict()
# u_media = dict()
# u_followers = dict()
# with open('/Users/cchen224/Downloads/instagram_media_users.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     for row in csvreader:
#         u_followers[row[0]] = row[2]
#         u_friends[row[0]] = row[3]
#         u_media[row[0]] = row[4]
# plt.hist([int(val) for val in u_followers.values()], bins=50)
# plt.hist([int(val) for val in u_followers.values() if int(val) < 5000], bins=50)
# plt.hist([int(val) for val in u_friends.values() if int(val) < 5000], bins=50)
# # plt.hist([float(u_followers[key]) / float(u_friends[key]) for key in u_media.keys() if float(u_friends[key]) != 0. and float(u_followers[key]) / float(u_friends[key]) < 5000], bins=50)
#
# post_perweek = dict()
# for i in range(len(users)):
#     key = users[i] + '@' + dt[i].strftime('%w')
#     if location_lat[i] != '':
#         if key not in post_perweek:
#             post_perweek[key] = 1
#         else:
#             post_perweek[key] += 1
#
#
# week_t = []
# week = []
# for i in range(len(users)):
#     week_t.append( dt[i].strftime('%d') )
#     if location_lat[i] != '':
#         week.append( dt[i].strftime('%d') )
# week = [int(day) for day in week]
# week_t = [int(day) for day in week_t]
# week_p = []
# for i in range(1,32):
#     week_p.append( float(len([item for item in week if item == i])) / len([item for item in week_t if item == i]))
# plt.plot(range(1,32), week_p)
# plt.xlabel('Day over month')
# plt.ylabel('Geocoded Percentage')
# plt.show()
#
# plt.hist(week_t, label='overall', bins=31, range=[1,32])
# plt.hist(week, label='geo-coded', bins=31, range=[1,32])
# plt.legend(loc='upper right')
# plt.show()
#
# users = dict()
# for item in uout: users[item] = 1
# for i in range(len(users)):
#     if users[i] in users:
#
#
#
#
#
# import pandas as pd
# data = pd.read_csv('/Users/cchen224/Downloads/locations.csv', header=None)
#
#
# import csv
# with open('/Users/cchen224/Downloads/instagram_media_users.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     with open('/Users/cchen224/Downloads/instagram_media.csv', 'w') as o:
#         csvwriter = csv.writer(o)
#         for row in csvreader:
#             if int(row[4]) > 20:
#                 csvwriter.writerow(row)


import json
import traceback
from random import random
from sys import stdout
from time import sleep

import requests
from cutils import Gmail


def parse_location(latlong, access_token):
    e_tmr = 0
    e_conn = 0
    e_is = 0

    url = get_url(latlong, access_token)
    jsontext = ''

    for counter in xrange(5):
        try:
            r = requests.get(url)
            jsontext = json.loads(r.text)

            e_tmr = 0
            e_conn = 0

            status = jsontext['meta'].get('code')

            if status == 200:
                e_is = 0
                venues = jsontext.get('response', {'venues': []}).get('venues', [])
                if venues:
                    venue = venues[0]
                    name = venue['name'].encode('utf-8')
                    city = venue['location'].get('city', '').encode('utf-8')
                    state = venue['location'].get('state', '').encode('utf-8')
                    zip = venue['location'].get('postalCode', '').encode('utf-8')
                    country = venue['location'].get('country', '').encode('utf-8')
                    distance = venue['location'].get('distance', '')
                    if venue['categories']:
                        category = venue['categories'][0].get('name', '').encode('utf-8')
                    else:
                        category = ''
                    checkinsCount = venue['stats'].get('checkinsCount', 0)
                    return [name, category, checkinsCount, zip, city, state, country, distance]
                else:
                    return []
            else:
                e_is += 1
                stdout(' InvalidStatus,')
                stdout(counter)
                stdout('\r')
                stdout.flush()
                if e_is >= 5:
                    g_error = Gmail().set_from_to().set_credentials().set_subject('AWS Foursquare APIError!')
                    g_error.send_message(latlong + '@' + str(access_token) + '\n\n' + jsontext['meta'].__str__()).close()
                    break
                else:
                    sleep(60)
                    continue

        except requests.ConnectionError:
            sleep(random() * 60)
            e_conn += 1
            if e_conn >= 5:
                g_error = Gmail().set_from_to().set_credentials().set_subject('AWS Foursquare ConnectionError!')
                g_error.send_message(latlong + '@' + str(access_token) + '\n\nConnectionError').close()
                return []
            else:
                continue

        except requests.TooManyRedirects:
            sleep(random() * 60)
            e_tmr += 1
            if e_tmr >= 5:
                g_error = Gmail().set_from_to().set_credentials().set_subject('AWS Foursquare TooManyRedirects!')
                g_error.send_message(latlong + '@' + str(access_token) + '\n\nTooManyRedirects').close()
                return []
            else:
                continue

        except:
            g_error = Gmail().set_from_to().set_credentials().set_subject('AWS Foursquare Error!')
            g_error.send_message(latlong + '@' + str(access_token))
            g_error.send_message(traceback.format_exc())
            g_error.send_message('\n\n\n')
            if jsontext:
                g_error.send_message(jsontext.__str__()).close()
            else:
                g_error.close()
            return []


def get_url(latlong, access_token):
    if ',' in access_token:
        token = access_token.split(',')
        clientid = token[0]
        clientsecret = token[1]
        return 'https://api.foursquare.com/v2/venues/search?ll=' + str(latlong) + '&query=&limit=1&client_id=' + clientid + '&client_secret=' + clientsecret +'&v=20151216'
    else:
        return 'https://api.foursquare.com/v2/venues/search?ll=' + str(latlong) + '&query=&limit=1&oauth_token=' + access_token + '&v=20151216'



access_tokens = ['RUBZJ00DZZYMHTSJV0QX05GYAPNGF4ACGUFWKALDZ2QZNFVJ,RRJZVFYOAXAR54MDLSIA01E0MGUFXXVFR3FUMV03WRCYTUV4',
                 'FKEOIYKASGWN4USCPG3VVAM2QC03JR4AWAVBJ0XKUFCPJZFD,5VSOXUL42WHLCBUAEEKBOL0P0WRIEPPSKYXUO5YJQLPQIWBY',
                 'KXEAGWMDSRZ2Y5MHMGUHAL0PD3MXX53DPOECVHWLMW42IFYM,2H3RTLHZV0M13LSKKTUG2ANZFL2HQQVRRXWYPBOBTL2SQBAU',
                 'TO5ROSEVPAPCVTN4YVSY2BAOIY0WATC5EV3W0WYXAY5FBFCB,PLI2RNVSL00TIER3G2ASUUYAA0L2FQAN0GBQUT3RX4IQOXWS',
                 'KSFFEW1DKVGPPV3FYVCCDHD2BAK42B0BZZ5UJYLK5F22LFCF,PXPW5QRVB412KIZXS5H4LGP3A4R0IMXNCDQQK0UCHWPVGQ34',
                 'Q4JML4D1GMHOCZM5JQL0KH3ED243ZADMLJAAYFBDBCLSWPXS,TS3ODKU4S1PRVGRQPWRNCAWYMS0KND123TYZ0T1YZIB12YG0',
                 '3P4ODDN3PDWCRRZ0D0IR4REHABQA1PLFFXTMYOJMR1AUZ0BV,B31V3XMQ0ADDZWI0NBQEIGZRLD3U2GZFBHC1PA5XTWRTYJU0',
                 'KOJNLSH3WHCUYLD2M3IFZHY4KX2F4C2UMPIBIISVWQS4ES2C,I5HJ2IHAJ1E5EURUVOEQAMWITM0JFEEA5UUPRFXKNJKPW4H2',
                 '4UAC1GRGKRVSNBSUZ35XMZTDOEML3H4IKGQ1RD3CSFESH5HU,QUWAYNZMMZKSVZZKTGDNXV43TSV2EEAZUCWJ2JN3R55LMUG5',
                 'R3P03EEUTG23MNJHOXERTRZCUSJ0KCQZAVB1YOZQCHOJVJMJ,O3RVROSNI3BKSDTVX2NIGNIXDHCLYMXRC3Z4NZYTYW1VGOKE',
                 'LL3DUGLBTIT3V4FDWQIYHLK0Z4ZK4CMMLAQCFS44H15CFXK0,50LJGTTRHNIWMOZATEKYPW2TFTT5FAIMOZILUWMWF3QEULUF',
                 'AQDUWBR4C40TTO23BQEUTTWJFFHDXZZ2HGBJMLCPUQRTBHWP,KITOBIXAW1HP5OE0ZCXXTKCGENIBSZSWWXI0VCKPQ0WOFBWN',
                 'SPVCJUS0BU1PEVI1WIOAZS2RZQ0ZXQEWLO1KKZA3UK0PVZBV,L2UVWY2DKFFDUG3GJJSBAG25TC2VCE5YFRQCOIGPNA4TVOUQ',
                 'JPZUOSPDUSIAMXEJ4LMWDEGSFQTPGFXXV2FGISGARNG1Z3WT,VUCTRJLVN0ODQZHTPTMK4O0DLU2GZVXKNTUFRK4G3WCCH2LN',
                 '1LAUQCBTYMOWN5KXNKBU4JOUKDUYDPSZIYREP0JXDMJRKFQ3,00POMCINTW3CZDEFKZMPPV3V5KEXP5P1B0YN2RK3NZRC44UE',
                 'MNKSG4UGN05ICLQR22XUZFDUM3P3JPUTQZNGOBGRW0TBJBXN,0PGRYKUICW3ZEZNMZ21QMATPRQC4INAFD4AK4JHIGODY2JDZ',
                 'IWFPWNTNZD2TNSN1S5EDVFMTXEIQWBSOS52ZUV4Y5H2OMXL2,CTYTIE2LTCLWRP2YJXWEEZXNQQH3UYMDO2MCEGKK0JN354SI',
                 '2DM00TG5KWM0ZZN3BBNOEQPGRPPEXZRL2RR2LUXIUXLTCVAG,42MUT100CFJ1ORBKSQ15XAP2OVC4GGIBHO25AAHITDKNKWJV',
                 'O0PUJQVNEYEHU5XW5JKXEVEI014E1ERCXNANTHWVIVEI4DSD,PJSDTPJRFAC5LIOSKDFTTOVBNNJWHXL13ZWLWX4IAILOT23L',
                 'YMCZUWEYCR4CT32CLU5BHCHDOC5IUJZJCTLNX3S04KROPMQ0,BLA5X3K3VQTSRJWPS1GHIT3YG14KM1QXPQDCXSFZ2V3PFZAK',

                 'MJBT1M2HM0ADCKK5MRQQOZOYBUMLSUTUBZ4S5TJ4F5SDP4MD',
                 'UTD22EWNQ0BRGWXB21350AKPYVKD55QSGTLHJ2QROTWJ4W4J',
                 'PSKFHUEZDQA131CJBFD5LQNNWTL35G2D4YTILXRIVH5BZJOP',]
len(access_tokens)


import csv
import time
import traceback

from bs4 import BeautifulSoup
from cutils import Gmail
from cutils import ProgressBar

from webparser_utility import *


def parse_user_info(user_id, **kwargs):
    """
    :param user_id:
    :return: user_id, is_verified, bio, location, join_date, tweets, following, followers, likes, profile_picture
    """
    has_profile_header = kwargs.get('header', True)
    has_profile_navigation = kwargs.get('navigation', True)
    has_profile_picture = kwargs.get('picture', True)

    url = 'https://twitter.com/' + str(user_id)
    html = get_html(url)
    soup = BeautifulSoup(html, 'html.parser')
    out = [user_id]
    if has_profile_header:
        out.extend(parse_profile_header(soup.find('div', class_='ProfileHeaderCard')))
    if has_profile_navigation:
        out.extend(parse_profile_navigation(soup.find('div', class_='ProfileNav')))
    if has_profile_picture:
        out.extend(parse_profile_picture(soup))
    return out


def parse_profile_header(profile_header_soup):
    try:
        bio = profile_header_soup.find('p', class_='ProfileHeaderCard-bio u-dir').text.strip()
        bio = re.sub('"|\n|\t', '', bio).encode('utf-8')
    except AttributeError:
        bio = ''
    try:
        location = profile_header_soup.find('div', class_='ProfileHeaderCard-location').text.strip().encode('utf-8')
    except AttributeError:
        location = ''
    try:
        join_date = profile_header_soup.find('div', class_='ProfileHeaderCard-joinDate').text.strip()
    except AttributeError:
        join_date = ''
    try:
        is_verified = profile_header_soup.h1.span.text.strip()
        if is_verified == 'Verified account':
            is_verified = 'yes'
        else:
            is_verified = 'no'
    except AttributeError:
        is_verified = 'no'

    return [is_verified, bio, location, join_date]


def parse_profile_navigation(profile_navigation_soup):
    try:
        tweets = profile_navigation_soup.find('li', class_='ProfileNav-item ProfileNav-item--tweets is-active').a.attrs['title']
    except AttributeError:
        tweets = str(0)
    try:
        following = profile_navigation_soup.find('li', class_='ProfileNav-item ProfileNav-item--following').a.attrs['title']
    except AttributeError:
        following = str(0)
    try:
        followers = profile_navigation_soup.find('li', class_='ProfileNav-item ProfileNav-item--followers').a.attrs['title']
    except AttributeError:
        followers = str(0)
    try:
        likes = profile_navigation_soup.find('li', class_='ProfileNav-item ProfileNav-item--favorites').a.attrs['title']
    except AttributeError:
        likes = str(0)

    tweets = re.sub(' Tweets| Tweet|,', '', tweets)
    following = re.sub(' Following|,', '', following)
    followers = re.sub(' Followers| Follower|,', '', followers)
    likes = re.sub(' Likes| Like|,', '', likes)
    return [tweets, following, followers, likes]


def parse_profile_picture(soup):
    pic_url = soup.find('img', class_='ProfileAvatar-image').attrs.get('src')
    return [pic_url.encode('utf-8')]


def crawl_user_info(input, output):
    gmail = Gmail().set_from_to().set_subject('AWS Twitter.info').send_message(input + '\n\n\n')
    print 'Starting...'

    if isinstance(input, str):
        with open(input, 'r') as i:
            users = [line.strip() for line in i]
    if isinstance(input, list):
        users = input

    bar = ProgressBar(total=len(users))
    with open(output, 'a') as o:
        csvwriter = csv.writer(o)
        for user in users:
            bar.move().log()
            if random.uniform(0, 1) > 0.999:
                time.sleep(random.uniform(4, 6))
            try:
                gmail = gmail.send_message(' User ' + user + '...')
                csvwriter.writerow(parse_user_info(user))
                gmail = gmail.send_message('Done!\n')
            except KeyboardInterrupt:
                gmail = gmail.send_message('KeyboardInterrupt.\n')
                break
            except:
                Gmail().set_from_to().set_subject('AWS Twitter.info Error') \
                    .send_message(input + ':user ' + user + '\n' + traceback.format_exc()).close()
        gmail.send_message('\n\n' + '\nDone').close()
    bar.close()



from setuptools import setup, find_packages

setup(
    name = 'cchen224-ec2',
    version = '0.0.110',
    keywords = ('ec2', 'cchen224'),
    description = 'cc personal use for AWS',
    license = 'MIT License',
    install_requires = ['cc.utils','tweepy','python-instagram','bs4', 'geopy', 'foursquare'],
    include_package_data=True,
    zip_safe=True,

    author = 'cchen224',
    author_email = 'phantomkidding@gmail.com',

    packages = find_packages(),
    platforms = 'any',
)